(function (global) {
  function Table() {
    this.colCount = 0;
    this.rowCount = Infinity;
    this.rows = [];
    this.title = "";
    this.headers = [];
  }

  var proto = Table.prototype;

  proto.setTitle = function (title) {
    this.title = title;
  };
  proto.setColumnCount = function (count) {
    this.colCount = count;
  };
  proto.setHeaderTitles = function (headerNames) {
    this.headers = headerNames;
  };
  proto.setRowCount = function (count) {
    this.rowCount = count;
  };

  proto.addRow = function () {
    this.rows.push(Array.prototype.slice.call(arguments));
  };
  proto.addRowFromString = function (str) {
    this.rows.push(str.split(";"));
  };
  proto.addRowsJson = function (data) {
    Array.prototype.push.apply(this.rows, data);
  };

  proto.createTable = function (id) {
    var el = document.getElementById(id);
    if (!el) {
      list.push(proto.createTable.bind(this, id));
      return;
    }

    var titleEl = document.createElement("h2");
    titleEl.innerHTML = this.title;
    el.appendChild(titleEl);

    var data = [];
    for (var i = 0; i < Math.min(this.rows.length, this.rowCount); ++i) {
      var row = [];
      for (var j = 0; j < this.colCount; ++j) {
        row.push(this.rows[i][j] || "");
      }
      data.push(row);
    }

    var
    table = document.createElement("table"),
    thead = document.createElement("thead"),
    tbody = document.createElement("tbody"),
    headerTr = document.createElement("tr");
    table.appendChild(thead);
    table.appendChild(tbody);
    thead.appendChild(headerTr);
    el.appendChild(table);

    table.border = 1;
    table.style.borderCollapse = "collapse";

    for (var i = 0; i < this.colCount; ++i) {
      var headerTh = document.createElement("th");
      headerTr.appendChild(headerTh);
      headerTh.innerHTML = this.headers[i] || "";
      headerTh.colIndex = i;
      headerTh.onmouseover = highlightCol;
      headerTh.onmouseout = unhighlightCol;
      headerTh.onclick = sort;
    }

    function highlightCol() {
      var col = cols[this.colIndex], color = "#d7e6f5";

      this.style.backgroundColor = color;
      for (var i = 0; i < col.length; ++i) {
        col[i].style.backgroundColor = color;
      }
    }
    function unhighlightCol() {
      var col = cols[this.colIndex], color = "";

      this.style.backgroundColor = color;
      for (var i = 0; i < col.length; ++i) {
        col[i].style.backgroundColor = color;
      }
    }

    function highlightRow() {
      var row = rows[this.rowIndex], color = "#d1f4da";

      for (var i = 0; i < row.length; ++i) {
        row[i].style.backgroundColor = color;
      }
    }
    function unhighlightRow() {
      var row = rows[this.rowIndex], color = "";

      for (var i = 0; i < row.length; ++i) {
        row[i].style.backgroundColor = color;
      }
    }

    var cols, rows;
    function clear() {
      while (tbody.firstChild) {
        tbody.removeChild(tbody.firstChild);
      }
    }
    function fill() {
      clear();
      cols = [];
      rows = [];

      for (var i = 0; i < data.length; ++i) {
        var row = data[i], tr = document.createElement("tr");
        tbody.appendChild(tr);

        rows[i] = [];

        for (var j = 0; j < row.length; ++j) {
          var item = row[j], td = document.createElement("td");
          tr.appendChild(td);
          td.innerHTML = item;
          td.rowIndex = i;
          td.colIndex = j;
          td.onmouseover = highlightRow;
          td.onmouseout = unhighlightRow;
          td.ondblclick = edit;

          (cols[j] = cols[j] || []).push(td);
          rows[i].push(td);
        }
      }
    }

    function edit() {
      var val = prompt("Nová hodnota", data[this.rowIndex][this.colIndex]);
      if (val !== null) {
        data[this.rowIndex][this.colIndex] = val;
        fill();
      }
    }

    var sortEl = document.createElement("span"), uarr = "&uarr;", darr = "&darr;";
    sortEl.style.paddingLeft = "3px";

    function sort() {
      var colIndex = this.colIndex;

      if (sortEl.parentNode === this && sortEl.sortType === darr) {
        sortEl.innerHTML = sortEl.sortType = uarr;
      }
      else {
        sortEl.innerHTML = sortEl.sortType = darr;
      }
      this.appendChild(sortEl);

      data.sort(
        sortEl.sortType === darr
        ? function (a, b) { return a[colIndex].localeCompare(b[colIndex], undefined, { numeric: true }); }
        : function (b, a) { return a[colIndex].localeCompare(b[colIndex], undefined, { numeric: true }); }
      );

      fill();
      highlightCol.call(this);
    }

    fill();
  };

  var list = [];
  window.onload = function () {
    for (var i = 0; i < list.length; ++i) {
      list[i]();
    }
  };

  // Psal jsem to jako třídu, pak jsem si všiml testovacího souboru. Tak tady máte jednu instanci.
  global.tables = new Table();
})(this);
